@@ -34,9 +34,8 @@ import os
 import sys
 import logging
 import uuid
-from pathlib import Path
-from distutils.version import LooseVersion as V
 from collections import namedtuple
+from collections import defaultdict
 
 import nbxmpp
 from gi.repository import Gdk
@@ -45,20 +44,23 @@ import gajim
 from gajim.common import config as c_config
 from gajim.common import configpaths
 from gajim.common import ged as ged_module
+from gajim.common.i18n import LANG
 from gajim.common.const import Display
-from gajim.common.contacts import LegacyContactsAPI
 from gajim.common.events import Events
 from gajim.common.types import NetworkEventsControllerT  # pylint: disable=unused-import
 from gajim.common.types import InterfaceT  # pylint: disable=unused-import
-from gajim.common.types import LoggerT  # pylint: disable=unused-import
 from gajim.common.types import ConnectionT  # pylint: disable=unused-import
+from gajim.common.types import LegacyContactsAPIT  # pylint: disable=unused-import
+from gajim.common.types import SettingsT  # pylint: disable=unused-import
 
 interface = cast(InterfaceT, None)
 thread_interface = lambda *args: None # Interface to run a thread and then a callback
 config = c_config.Config()
+settings = cast(SettingsT, None)
 version = gajim.__version__
 connections = {}  # type: Dict[str, ConnectionT]
 avatar_cache = {}  # type: Dict[str, Dict[str, Any]]
+bob_cache = {} # type: Dict[str, bytes]
 ipython_window = None
 app = None  # Gtk.Application
 
@@ -66,26 +68,22 @@ ged = ged_module.GlobalEventsDispatcher() # Global Events Dispatcher
 nec = cast(NetworkEventsControllerT, None)
 plugin_manager = None # Plugins Manager
 
-logger = cast(LoggerT, None)
+class Storage:
+    def __init__(self):
+        self.cache = None
+        self.archive = None
 
-# For backwards compatibility needed
-# some plugins use that
-gajimpaths = configpaths.gajimpaths
-
-
-RecentGroupchat = namedtuple('RecentGroupchat', ['room', 'server', 'nickname'])
+storage = Storage()
 
 css_config = None
 
-os_info = None # used to cache os information
-
 transport_type = {}  # type: Dict[str, str]
 
 # dict of time of the latest incoming message per jid
 # {acct1: {jid1: time1, jid2: time2}, }
 last_message_time = {}  # type: Dict[str, Dict[str, float]]
 
-contacts = LegacyContactsAPI()
+contacts = cast(LegacyContactsAPIT, None)
 
 # tell if we are connected to the room or not
 # {acct: {room_jid: True}}
@@ -95,7 +93,7 @@ gc_connected = {}  # type: Dict[str, Dict[str, bool]]
 # {room_jid: password}
 gc_passwords = {}  # type: Dict[str, str]
 
-# dict of rooms that must be automaticaly configured
+# dict of rooms that must be automatically configured
 # and for which we have a list of invities
 # {account: {room_jid: {'invities': []}}}
 automatic_rooms = {}  # type: Dict[str, Dict[str, Dict[str, List[str]]]]
@@ -123,22 +121,11 @@ nicks = {}  # type: Dict[str, str]
 # from this transport
 block_signed_in_notifications = {}  # type: Dict[str, bool]
 
- # type of each connection (ssl, tls, tcp, ...)
-con_types = {}  # type: Dict[str, Optional[str]]
+proxy65_manager = None
 
-# whether we pass auto away / xa or not
-#'off': don't use sleeper for this account
-#'online': online and use sleeper
-#'autoaway': autoaway and use sleeper
-#'autoxa': autoxa and use sleeper
-sleeper_state = {}  # type: Dict[str, str]
+cert_store = None
 
-status_before_autoaway = {}  # type: Dict[str, str]
-
-# Is Gnome configured to activate on single click ?
-single_click = False
-SHOW_LIST = ['offline', 'connecting', 'online', 'chat', 'away', 'xa', 'dnd',
-             'invisible', 'error']
+task_manager = None
 
 # zeroconf account name
 ZEROCONF_ACC_NAME = 'Local'
@@ -149,52 +136,46 @@ socks5queue = None
 
 gupnp_igd = None
 
-gajim_identity = {'type': 'pc', 'category': 'client', 'name': 'Gajim'}
-
-gajim_common_features = [nbxmpp.NS_BYTESTREAM, nbxmpp.NS_SI, nbxmpp.NS_FILE,
-    nbxmpp.NS_MUC, nbxmpp.NS_MUC_USER, nbxmpp.NS_MUC_ADMIN, nbxmpp.NS_MUC_OWNER,
-    nbxmpp.NS_MUC_CONFIG, nbxmpp.NS_COMMANDS, nbxmpp.NS_DISCO_INFO, 'ipv6',
-    'jabber:iq:gateway', nbxmpp.NS_LAST, nbxmpp.NS_PRIVACY, nbxmpp.NS_PRIVATE,
-    nbxmpp.NS_REGISTER, nbxmpp.NS_VERSION, nbxmpp.NS_DATA, nbxmpp.NS_ENCRYPTED,
-    'msglog', 'sslc2s', 'stringprep', nbxmpp.NS_PING, nbxmpp.NS_TIME_REVISED,
-    nbxmpp.NS_SSN, nbxmpp.NS_MOOD, nbxmpp.NS_ACTIVITY, nbxmpp.NS_NICK,
-    nbxmpp.NS_ROSTERX, nbxmpp.NS_SECLABEL, nbxmpp.NS_HASHES_2,
-    nbxmpp.NS_HASHES_MD5, nbxmpp.NS_HASHES_SHA1, nbxmpp.NS_HASHES_SHA256,
-    nbxmpp.NS_HASHES_SHA512, nbxmpp.NS_CONFERENCE, nbxmpp.NS_CORRECT,
-    nbxmpp.NS_EME, 'urn:xmpp:avatar:metadata+notify']
-
-# Optional features gajim supports per account
-gajim_optional_features = {}  # type: Dict[str, List[str]]
-
-# Capabilities hash per account
-caps_hash = {}  # type: Dict[str, List[str]]
+gsound_ctx = None
 
 _dependencies = {
-    'PYTHON-DBUS': False,
+    'AVAHI': False,
     'PYBONJOUR': False,
-    'PYGPG': False,
-    'GPG_BINARY': False,
     'FARSTREAM': False,
+    'GST': False,
+    'AV': False,
     'GEOCLUE': False,
     'UPNP': False,
-    'PYCURL': False,
+    'GSOUND': False,
     'GSPELL': False,
     'IDLE': False,
 }
 
+_tasks = defaultdict(list)  # type: Dict[int, List[Any]]
+
+def print_version():
+    log('gajim').info('Gajim Version: %s', gajim.__version__)
+
+
+def get_client(account):
+    return connections[account]
+
 
 def is_installed(dependency):
-    if dependency == 'GPG':
-        # Alias for checking python-gnupg and the GPG binary
-        return _dependencies['PYGPG'] and _dependencies['GPG_BINARY']
     if dependency == 'ZEROCONF':
         # Alias for checking zeroconf libs
-        return _dependencies['PYTHON-DBUS'] or _dependencies['PYBONJOUR']
+        return _dependencies['AVAHI'] or _dependencies['PYBONJOUR']
     return _dependencies[dependency]
 
+
 def is_flatpak():
     return gajim.IS_FLATPAK
 
+
+def is_portable():
+    return gajim.IS_PORTABLE
+
+
 def is_display(display):
     # XWayland reports as Display X11, so try with env var
     is_wayland = os.environ.get('XDG_SESSION_TYPE') == 'wayland'
@@ -215,77 +196,45 @@ def detect_dependencies():
 
     # ZEROCONF
     try:
-        import pybonjour  # pylint: disable=unused-variable
+        import pybonjour  # pylint: disable=unused-import
         _dependencies['PYBONJOUR'] = True
     except Exception:
         pass
 
     try:
-        import dbus  # pylint: disable=unused-variable
-        _dependencies['PYTHON-DBUS'] = True
+        gi.require_version('Avahi', '0.6')
+        from gi.repository import Avahi  # pylint: disable=unused-import
+        _dependencies['AVAHI'] = True
     except Exception:
         pass
 
-    # python-gnupg
     try:
-        import gnupg
-        # We need https://pypi.python.org/pypi/python-gnupg
-        # but https://pypi.python.org/pypi/gnupg shares the same package name.
-        # It cannot be used as a drop-in replacement.
-        # We test with a version check if python-gnupg is installed as it is
-        # on a much lower version number than gnupg
-        # Also we need at least python-gnupg 0.3.8
-        v_gnupg = gnupg.__version__
-        if V(v_gnupg) < V('0.3.8') or V(v_gnupg) > V('1.0.0'):
-            log('gajim').info('Gajim needs python-gnupg >= 0.3.8')
-            raise ImportError
-        _dependencies['PYGPG'] = True
-    except ImportError:
-        pass
-
-    # GPG BINARY
-    import subprocess
-
-    def test_gpg(binary='gpg'):
-        if os.name == 'nt':
-            gpg_cmd = binary + ' -h >nul 2>&1'
-        else:
-            gpg_cmd = binary + ' -h >/dev/null 2>&1'
-        if subprocess.call(gpg_cmd, shell=True):
-            return False
-        return True
-
-    if test_gpg(binary='gpg2'):
-        _dependencies['GPG_BINARY'] = 'gpg2'
-    elif test_gpg(binary='gpg'):
-        _dependencies['GPG_BINARY'] = 'gpg'
-
-    # FARSTREAM
-    try:
-        if os.name == 'nt':
-            root_path = Path(sys.executable).parents[1]
-            fs_plugin_path = str(root_path / 'lib' / 'farstream-0.2')
-            gst_plugin_path = str(root_path / 'lib' / 'gstreamer-1.0')
-            os.environ['FS_PLUGIN_PATH'] = fs_plugin_path
-            os.environ['GST_PLUGIN_PATH'] = gst_plugin_path
-        gi.require_version('Farstream', '0.2')
-        from gi.repository import Farstream
         gi.require_version('Gst', '1.0')
         from gi.repository import Gst
-        try:
+        _dependencies['GST'] = True
+    except Exception:
+        pass
+
+    try:
+        gi.require_version('Farstream', '0.2')
+        from gi.repository import Farstream
+        _dependencies['FARSTREAM'] = True
+    except Exception:
+        pass
+
+    try:
+        if _dependencies['GST'] and _dependencies['FARSTREAM']:
             Gst.init(None)
             conference = Gst.ElementFactory.make('fsrtpconference', None)
             conference.new_session(Farstream.MediaType.AUDIO)
-        except Exception as error:
-            log('gajim').info(error)
-        _dependencies['FARSTREAM'] = True
-    except (ImportError, ValueError):
-        pass
+            _dependencies['AV'] = True
+    except Exception as error:
+        log('gajim').warning('AV dependency test failed: %s', error)
 
     # GEOCLUE
     try:
         gi.require_version('Geoclue', '2.0')
-        from gi.repository import Geoclue  # pylint: disable=unused-variable
+        from gi.repository import Geoclue  # pylint: disable=unused-import
         _dependencies['GEOCLUE'] = True
     except (ImportError, ValueError):
         pass
@@ -300,13 +249,6 @@ def detect_dependencies():
     except ValueError:
         pass
 
-    # PYCURL
-    try:
-        import pycurl  # pylint: disable=unused-variable
-        _dependencies['PYCURL'] = True
-    except ImportError:
-        pass
-
     # IDLE
     try:
         from gajim.common import idle
@@ -315,6 +257,21 @@ def detect_dependencies():
     except Exception:
         pass
 
+    # GSOUND
+    try:
+        gi.require_version('GSound', '1.0')
+        from gi.repository import GLib
+        from gi.repository import GSound
+        global gsound_ctx
+        gsound_ctx = GSound.Context()
+        try:
+            gsound_ctx.init()
+            _dependencies['GSOUND'] = True
+        except GLib.Error as error:
+            log('gajim').warning('GSound init failed: %s', error)
+    except (ImportError, ValueError):
+        pass
+
     # GSPELL
     try:
         gi.require_version('Gspell', '1')
@@ -332,8 +289,21 @@ def detect_dependencies():
     for dep, val in _dependencies.items():
         log('gajim').info('%-13s %s', dep, val)
 
-def get_gpg_binary():
-    return _dependencies['GPG_BINARY']
+    log('gajim').info('Used language: %s', LANG)
+
+def detect_desktop_env():
+    if sys.platform in ('win32', 'darwin'):
+        return sys.platform
+
+    desktop = os.environ.get('XDG_CURRENT_DESKTOP')
+    if desktop is None:
+        return None
+
+    if 'gnome' in desktop.lower():
+        return 'gnome'
+    return desktop
+
+desktop_env = detect_desktop_env()
 
 def get_an_id():
     return str(uuid.uuid4())
@@ -419,16 +389,36 @@ def get_number_of_connected_accounts(accounts_list=None):
             connected_accounts = connected_accounts + 1
     return connected_accounts
 
-def get_connected_accounts():
+def get_available_clients():
+    clients = []
+    for client in connections.values():
+        if client.state.is_available:
+            clients.append(client)
+    return clients
+
+def get_connected_accounts(exclude_local=False):
     """
     Returns a list of CONNECTED accounts
     """
     account_list = []
     for account in connections:
+        if account == 'Local' and exclude_local:
+            continue
         if account_is_connected(account):
             account_list.append(account)
     return account_list
 
+def get_accounts_sorted():
+    '''
+    Get all accounts alphabetically sorted with Local first
+    '''
+    account_list = settings.get_accounts()
+    account_list.sort(key=str.lower)
+    if 'Local' in account_list:
+        account_list.remove('Local')
+        account_list.insert(0, 'Local')
+    return account_list
+
 def get_enabled_accounts_with_labels(exclude_local=True, connected_only=False,
                                      private_storage_only=False):
     """
@@ -450,7 +440,7 @@ def get_enabled_accounts_with_labels(exclude_local=True, connected_only=False,
     return accounts
 
 def get_account_label(account):
-    return config.get_per('accounts', account, 'account_label') or account
+    return settings.get_account_setting(account, 'account_label') or account
 
 def account_is_zeroconf(account):
     return connections[account].is_zeroconf
@@ -463,39 +453,27 @@ def account_supports_private_storage(account):
 def account_is_connected(account):
     if account not in connections:
         return False
-    # 0 is offline, 1 is connecting
-    return connections[account].connected > 1
+    return (connections[account].state.is_connected or
+            connections[account].state.is_available)
 
-def is_invisible(account):
-    return SHOW_LIST[connections[account].connected] == 'invisible'
+def account_is_available(account):
+    if account not in connections:
+        return False
+    return connections[account].state.is_available
 
 def account_is_disconnected(account):
     return not account_is_connected(account)
 
 def zeroconf_is_connected():
     return account_is_connected(ZEROCONF_ACC_NAME) and \
-            config.get_per('accounts', ZEROCONF_ACC_NAME, 'is_zeroconf')
+            settings.get_account_setting(ZEROCONF_ACC_NAME, 'is_zeroconf')
 
 def in_groupchat(account, room_jid):
+    room_jid = str(room_jid)
     if room_jid not in gc_connected[account]:
         return False
     return gc_connected[account][room_jid]
 
-def get_number_of_securely_connected_accounts():
-    """
-    Return the number of the accounts that are SSL/TLS connected
-    """
-    num_of_secured = 0
-    for account in connections:
-        if account_is_securely_connected(account):
-            num_of_secured += 1
-    return num_of_secured
-
-def account_is_securely_connected(account):
-    if not account_is_connected(account):
-        return False
-    return con_types.get(account) in ('tls', 'ssl')
-
 def get_transport_name_from_jid(jid, use_config_setting=True):
     """
     Returns 'gg', 'irc' etc
@@ -538,13 +516,13 @@ def get_jid_from_account(account_name):
     """
     Return the jid we use in the given account
     """
-    name = config.get_per('accounts', account_name, 'name')
-    hostname = config.get_per('accounts', account_name, 'hostname')
+    name = settings.get_account_setting(account_name, 'name')
+    hostname = settings.get_account_setting(account_name, 'hostname')
     jid = name + '@' + hostname
     return jid
 
 def get_account_from_jid(jid):
-    for account in config.get_per('accounts'):
+    for account in settings.get_accounts():
         if jid == get_jid_from_account(account):
             return account
 
@@ -563,9 +541,9 @@ def get_hostname_from_account(account_name, use_srv=False):
     """
     if use_srv and connections[account_name].connected_hostname:
         return connections[account_name].connected_hostname
-    if config.get_per('accounts', account_name, 'use_custom_host'):
-        return config.get_per('accounts', account_name, 'custom_host')
-    return config.get_per('accounts', account_name, 'hostname')
+    if settings.get_account_setting(account_name, 'use_custom_host'):
+        return settings.get_account_setting(account_name, 'custom_host')
+    return settings.get_account_setting(account_name, 'hostname')
 
 def get_notification_image_prefix(jid):
     """
@@ -589,24 +567,24 @@ def get_name_from_jid(account, jid):
         actor = jid
     return actor
 
-def get_muc_domain(account):
-    return connections[account].muc_jid.get('jabber', None)
 
 def get_recent_groupchats(account):
-    recent_groupchats = config.get_per(
-        'accounts', account, 'recent_groupchats').split()
+    recent_groupchats = settings.get_account_setting(
+        account, 'recent_groupchats').split()
+
+    RecentGroupchat = namedtuple('RecentGroupchat',
+                                 ['room', 'server', 'nickname'])
 
     recent_list = []
     for groupchat in recent_groupchats:
-        jid = nbxmpp.JID(groupchat)
-        recent = RecentGroupchat(
-            jid.getNode(), jid.getDomain(), jid.getResource())
+        jid = nbxmpp.JID.from_string(groupchat)
+        recent = RecentGroupchat(jid.localpart, jid.domain, jid.resource)
         recent_list.append(recent)
     return recent_list
 
 def add_recent_groupchat(account, room_jid, nickname):
-    recent = config.get_per(
-        'accounts', account, 'recent_groupchats').split()
+    recent = settings.get_account_setting(
+        account, 'recent_groupchats').split()
     full_jid = room_jid + '/' + nickname
     if full_jid in recent:
         recent.remove(full_jid)
@@ -614,8 +592,7 @@ def add_recent_groupchat(account, room_jid, nickname):
     if len(recent) > 10:
         recent = recent[0:9]
     config_value = ' '.join(recent)
-    config.set_per(
-        'accounts', account, 'recent_groupchats', config_value)
+    settings.set_account_setting(account, 'recent_groupchats', config_value)
 
 def get_priority(account, show):
     """
@@ -624,11 +601,11 @@ def get_priority(account, show):
     if not show:
         show = 'online'
 
-    if show in ('online', 'chat', 'away', 'xa', 'dnd', 'invisible') and \
-    config.get_per('accounts', account, 'adjust_priority_with_status'):
-        prio = config.get_per('accounts', account, 'autopriority_' + show)
+    if show in ('online', 'chat', 'away', 'xa', 'dnd') and \
+    settings.get_account_setting(account, 'adjust_priority_with_status'):
+        prio = settings.get_account_setting(account, 'autopriority_' + show)
     else:
-        prio = config.get_per('accounts', account, 'priority')
+        prio = settings.get_account_setting(account, 'priority')
     if prio < -128:
         prio = -128
     elif prio > 127:
@@ -647,28 +624,13 @@ def prefers_app_menu():
         return False
     return app.prefers_app_menu()
 
-def get_app_window(cls, account=None):
-    for win in app.get_windows():
-        if isinstance(cls, str):
-            if type(win).__name__ == cls:
-                if account is not None:
-                    if account != win.account:
-                        continue
-                return win
-        elif isinstance(win, cls):
-            if account is not None:
-                if account != win.account:
-                    continue
-            return win
-    return None
-
 def load_css_config():
     global css_config
-    from gajim.gtk.css_config import CSSConfig
+    from gajim.gui.css_config import CSSConfig
     css_config = CSSConfig()
 
-def set_win_debug_mode(enable: bool) -> None:
-    debug_folder = Path(configpaths.get('DEBUG'))
+def set_debug_mode(enable: bool) -> None:
+    debug_folder = configpaths.get('DEBUG')
     debug_enabled = debug_folder / 'debug-enabled'
     if enable:
         debug_enabled.touch()
@@ -676,9 +638,51 @@ def set_win_debug_mode(enable: bool) -> None:
         if debug_enabled.exists():
             debug_enabled.unlink()
 
-def get_win_debug_mode() -> bool:
-    if sys.platform != 'win32':
-        return False
-    debug_folder = Path(configpaths.get('DEBUG'))
+def get_debug_mode() -> bool:
+    debug_folder = configpaths.get('DEBUG')
     debug_enabled = debug_folder / 'debug-enabled'
     return debug_enabled.exists()
+
+def get_stored_bob_data(algo_hash: str) -> Optional[bytes]:
+    try:
+        return bob_cache[algo_hash]
+    except KeyError:
+        filepath = configpaths.get('BOB') / algo_hash
+        if filepath.exists():
+            with open(str(filepath), 'r+b') as file:
+                data = file.read()
+            return data
+    return None
+
+def get_groupchat_control(account, jid):
+    control = app.interface.msg_win_mgr.get_gc_control(jid, account)
+    if control is not None:
+        return control
+    try:
+        return app.interface.minimized_controls[account][jid]
+    except Exception:
+        return None
+
+
+def register_task(self, task):
+    _tasks[id(self)].append(task)
+
+
+def remove_task(task, id_):
+    try:
+        _tasks[id_].remove(task)
+    except Exception:
+        pass
+    else:
+        if not _tasks[id_]:
+            del _tasks[id_]
+
+
+def cancel_tasks(obj):
+    id_ = id(obj)
+    if id_ not in _tasks:
+        return
+
+    task_list = _tasks[id_]
+    for task in task_list:
+        task.cancel()
