@@ -15,35 +15,40 @@
 from gi.repository import Gdk
 from gi.repository import Gtk
 
+from gajim import vcard
 from gajim.common import app
 from gajim.common import ged
 from gajim.common import helpers
 from gajim.common.i18n import _
 
-from gajim.gtk.dialogs import ErrorDialog
-from gajim.gtk.util import get_builder
+from .dialogs import ErrorDialog
+from .util import get_builder
+from .util import EventHelper
 
 
-class AddNewContactWindow(Gtk.ApplicationWindow):
+class AddNewContactWindow(Gtk.ApplicationWindow, EventHelper):
 
-    uid_labels = {'jabber': _('Jabber ID'),
+    uid_labels = {'jabber': _('XMPP Address'),
                   'gadu-gadu': _('GG Number'),
                   'icq': _('ICQ Number')}
 
-    def __init__(self, account=None, jid=None, user_nick=None, group=None):
+    def __init__(self, account=None, contact_jid=None, user_nick=None, group=None):
         Gtk.ApplicationWindow.__init__(self)
+        EventHelper.__init__(self)
         self.set_application(app.app)
         self.set_position(Gtk.WindowPosition.CENTER)
         self.set_show_menubar(False)
         self.set_resizable(False)
         self.set_title(_('Add Contact'))
 
-        self.connect('destroy', self._on_destroy)
-        self.connect('key-press-event', self._on_key_press)
+        self.connect_after('key-press-event', self._on_key_press)
 
         self.account = account
         self.adding_jid = False
 
+        if contact_jid is not None:
+            contact_jid = app.get_jid_without_resource(contact_jid)
+
         # fill accounts with active accounts
         accounts = app.get_enabled_accounts_with_labels()
 
@@ -58,8 +63,9 @@ class AddNewContactWindow(Gtk.ApplicationWindow):
         self.xml.connect_signals(self)
 
         for w in ('account_combobox', 'account_label', 'prompt_label',
-                  'uid_label', 'uid_entry', 'protocol_combobox',
-                  'protocol_jid_combobox', 'protocol_label', 'nickname_entry',
+                  'uid_label', 'uid_entry', 'show_contact_info_button',
+                  'protocol_combobox', 'protocol_jid_combobox',
+                  'protocol_label', 'nickname_entry',
                   'message_scrolledwindow', 'save_message_checkbutton',
                   'register_hbox', 'add_button', 'message_textview',
                   'connected_label', 'group_comboboxentry',
@@ -68,6 +74,7 @@ class AddNewContactWindow(Gtk.ApplicationWindow):
             self.__dict__[w] = self.xml.get_object(w)
 
         self.subscription_table = [self.uid_label, self.uid_entry,
+                                   self.show_contact_info_button,
                                    self.nickname_label, self.nickname_entry,
                                    self.group_label, self.group_comboboxentry]
 
@@ -115,21 +122,25 @@ class AddNewContactWindow(Gtk.ApplicationWindow):
 
             if account:
                 for service in self.agents[type_]:
-                    app.connections[account].request_gateway_prompt(service)
+                    con = app.connections[account]
+                    con.get_module('Gateway').request_gateway_prompt(service)
         self.protocol_combobox.set_active(0)
         self.auto_authorize_checkbutton.show()
 
-        if jid:
+        if contact_jid:
             self.jid_escaped = True
-            type_ = app.get_transport_name_from_jid(jid)
+            type_ = app.get_transport_name_from_jid(contact_jid)
             if not type_:
                 type_ = 'jabber'
             if type_ == 'jabber':
-                self.uid_entry.set_text(jid)
+                self.uid_entry.set_text(contact_jid)
                 transport = None
             else:
-                uid, transport = app.get_name_and_server_from_jid(jid)
+                uid, transport = app.get_name_and_server_from_jid(contact_jid)
                 self.uid_entry.set_text(uid.replace('%', '@', 1))
+
+            self.show_contact_info_button.set_sensitive(True)
+
             # set protocol_combobox
             model = self.protocol_combobox.get_model()
             iter_ = model.get_iter_first()
@@ -190,24 +201,39 @@ class AddNewContactWindow(Gtk.ApplicationWindow):
                 self.account))
             message_buffer.set_text(msg)
 
+        self.uid_entry.connect('changed', self.on_uid_entry_changed)
         self.show_all()
 
-        app.ged.register_event_handler('gateway-prompt-received', ged.GUI1,
-                                       self._nec_gateway_prompt_received)
-        app.ged.register_event_handler('presence-received', ged.GUI1,
-                                       self._nec_presence_received)
+        self.register_events([
+            ('gateway-prompt-received', ged.GUI1, self._nec_gateway_prompt_received),
+            ('presence-received', ged.GUI1, self._nec_presence_received),
+        ])
 
-    def _on_destroy(self, widget):
-        app.ged.remove_event_handler('presence-received', ged.GUI1,
-                                     self._nec_presence_received)
-        app.ged.remove_event_handler('gateway-prompt-received', ged.GUI1,
-                                     self._nec_gateway_prompt_received)
+    def on_uid_entry_changed(self, widget):
+        is_empty = bool(not self.uid_entry.get_text() == '')
+        self.show_contact_info_button.set_sensitive(is_empty)
+
+    def on_show_contact_info_button_clicked(self, widget):
+        """
+        Ask for vCard
+        """
+        jid = self.uid_entry.get_text().strip()
+
+        if jid in app.interface.instances[self.account]['infos']:
+            app.interface.instances[self.account]['infos'][jid].window.present()
+        else:
+            contact = app.contacts.create_contact(jid=jid, account=self.account)
+            app.interface.instances[self.account]['infos'][jid] = \
+                     vcard.VcardWindow(contact, self.account)
+            # Remove xmpp page
+            app.interface.instances[self.account]['infos'][jid].xml.\
+                     get_object('information_notebook').remove_page(0)
 
     def on_register_button_clicked(self, widget):
         model = self.protocol_jid_combobox.get_model()
         row = self.protocol_jid_combobox.get_active()
         jid = model[row][0]
-        from gajim.gtk.service_registration import ServiceRegistration
+        from .service_registration import ServiceRegistration
         ServiceRegistration(self.account, jid)
 
     def _on_key_press(self, widget, event):
@@ -232,7 +258,7 @@ class AddNewContactWindow(Gtk.ApplicationWindow):
         if not jid:
             ErrorDialog(
                 _('%s Missing') % self.uid_label.get_text(),
-                _('You must supply the %s of the new contact.' %
+                (_('You must supply the %s of the new contact.') %
                     self.uid_label.get_text())
             )
             return
@@ -246,8 +272,8 @@ class AddNewContactWindow(Gtk.ApplicationWindow):
             transport = model[row][0]
             if self.account and not self.jid_escaped:
                 self.adding_jid = (jid, transport, type_)
-                app.connections[self.account].request_gateway_prompt(
-                    transport, jid)
+                con = app.connections[self.account]
+                con.get_module('Gateway').request_gateway_prompt(transport, jid)
             else:
                 jid = jid.replace('@', '%') + '@' + transport
                 self._add_jid(jid, type_)
@@ -271,10 +297,10 @@ class AddNewContactWindow(Gtk.ApplicationWindow):
 
         if jid == app.get_jid_from_account(self.account):
             pritext = _('Invalid User ID')
-            ErrorDialog(pritext, _('You cannot add yourself to your roster.'))
+            ErrorDialog(pritext, _('You cannot add yourself to your contact list.'))
             return
 
-        if not app.account_is_connected(self.account):
+        if not app.account_is_available(self.account):
             ErrorDialog(
                 _('Account Offline'),
                 _('Your account must be online to add new contacts.')
@@ -291,10 +317,10 @@ class AddNewContactWindow(Gtk.ApplicationWindow):
         # Check if jid is already in roster
         if jid in app.contacts.get_jid_list(self.account):
             c = app.contacts.get_first_contact_from_jid(self.account, jid)
-            if _('Not in Roster') not in c.groups and c.sub in ('both', 'to'):
+            if _('Not in contact list') not in c.groups and c.sub in ('both', 'to'):
                 ErrorDialog(
-                    _('Contact already in roster'),
-                    _('This contact is already listed in your roster.'))
+                    _('Contact Already in Contact List'),
+                    _('This contact is already in your contact list.'))
                 return
 
         if type_ == 'jabber':
@@ -304,8 +330,9 @@ class AddNewContactWindow(Gtk.ApplicationWindow):
             message = message_buffer.get_text(start_iter, end_iter, True)
             if self.save_message_checkbutton.get_active():
                 msg = helpers.to_one_line(message)
-                app.config.set_per('accounts', self.account,
-                                   'subscription_request_msg', msg)
+                app.settings.set_account_setting(self.account,
+                                                 'subscription_request_msg',
+                                                 msg)
         else:
             message = ''
         group = self.group_comboboxentry.get_child().get_text()
@@ -457,7 +484,7 @@ class AddNewContactWindow(Gtk.ApplicationWindow):
             if obj.stanza.getError():
                 ErrorDialog(
                     _('Error while adding transport contact'),
-                    _('This error occured while adding a contact for transport '
+                    _('This error occurred while adding a contact for transport '
                       '%(transport)s:\n\n%(error)s') % {
                         'transport': transport,
                         'error': obj.stanza.getErrorMsg()})
