@@ -9,7 +9,10 @@ from nbxmpp.modules.dataforms import SimpleDataForm
 
 from gajim.common.const import CSSPriority
 
-from gajim.gtk.change_password import ChangePassword
+from gajim import gui
+gui.init('gtk')
+
+from gajim.gui.change_password import ChangePassword
 
 from test.gtk import util
 util.load_style('gajim.css', CSSPriority.APPLICATION)
