@@ -1,6 +1,9 @@
 from gi.repository import Gtk
 
-from gajim.gtk.dataform import FakeDataFormWidget
+from gajim import gui
+gui.init('gtk')
+
+from gajim.gui.dataform import FakeDataFormWidget
 from gajim.common.const import CSSPriority
 
 from test.gtk import util
